@@ -1,5 +1,5 @@
 ;; custom.jl -- Emacs-like ``customizing'' (but more simple)
-;; $Id: custom.jl,v 1.61 2001/04/11 21:48:37 jsh Exp $
+;; $Id: custom.jl,v 1.64 2002/11/03 10:53:59 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -31,6 +31,7 @@
 	    custom-quote-keys
 	    define-custom-setter
 	    custom-set-property
+	    custom-get-property
 	    custom-set-group-property
 	    custom-get-group-property
 	    custom-add-option
@@ -85,6 +86,7 @@
 				(:type* . custom-type)
 				(:options . custom-options)
 				(:depends . custom-depends)
+				;; XXX not used anymore
 				(:user-level . custom-user-level)
 				(:set . custom-set)
 				(:get . custom-get)
@@ -115,7 +117,6 @@ KEYS is a property-list containing any of the following:
 	:type TYPE
 	:options OPTIONS
 	:depends SYMBOL
-	:user-level LEVEL		novice, intermediate, expert
 	:range (MIN . MAX)		for `number' type
 	:set FUNCTION
 	:get FUNCTION
@@ -127,8 +128,8 @@ TYPE may be `boolean', `number', `string', `symbol', `file-name',
 `program-name', `font', `color'.
 
 Note that the values of the `:group', `:require', `:type', `:options',
-`:depends', `:user-level' and `:range' keys are not evaluated. All
-other key values are evaluated.
+`:depends' and `:range' keys are not evaluated. All other key values are
+evaluated.
 
 Each defcustom'd symbol may have several special properties
 
@@ -233,12 +234,15 @@ Note that the value of the `:group' key is not evaluated."
     (or (table-ref custom-setter-table name)
 	(error "No such custom setter: %s" name)))
       
-  (defmacro custom-set-property (sym prop value)
+  (define (custom-set-property sym prop value)
     "Set the custom key PROP for defcustom'd symbol SYM to value."
-    (let ((tem (gensym)))
-      `(let ((,tem (cdr (assq ,prop custom-option-alist))))
-	 (when ,tem
-	   (put ,sym ,tem ,value)))))
+    (let ((key (cdr (assq prop custom-option-alist))))
+      (when key
+	(put sym key value))))
+
+  (define (custom-get-property sym prop)
+    (let ((key (cdr (assq prop custom-option-alist))))
+      (and key (get sym key))))
 
   (define (custom-set-group-property group prop value)
     "Set the custom key PROP for defgroup'd symbol SYM to value."
@@ -321,17 +325,18 @@ of choices."
       ((get symbol 'custom-after-set) symbol)))
 
   (define (custom-set-symbol setter symbol)
-    (let* ((was-bound (boundp symbol))
-	   (old-value (and was-bound (symbol-value symbol))))
-      (call-with-exception-handler
-       (lambda ()
-	 (custom-set setter symbol))
-       (lambda (ex)
-	 ;; error while setting SYMBOL; revert to its old state
-	 (if was-bound
-	     (set symbol old-value)
-	   (makunbound symbol))
-	 (raise-exception ex)))))
+    (unless (get symbol 'custom-obsolete)
+      (let* ((was-bound (boundp symbol))
+	     (old-value (and was-bound (symbol-value symbol))))
+	(call-with-exception-handler
+	 (lambda ()
+	   (custom-set setter symbol))
+	 (lambda (ex)
+	   ;; error while setting SYMBOL; revert to its old state
+	   (if was-bound
+	       (set symbol old-value)
+	     (makunbound symbol))
+	   (raise-exception ex))))))
  
   (define (custom-set-variable symbol value #!optional req)
     ;; XXX kludge for old custom files..
@@ -455,13 +460,17 @@ of choices."
 
   (define-custom-serializer 'font (lambda (value)
 				    (if (fontp value)
-					(font-name value)
+					(cons (font-type value)
+					      (font-name value))
 				      value)))
 
   (define-custom-deserializer 'font (lambda (value)
-				      (if (stringp value)
-					  (get-font value)
-					value)))
+				      (cond ((stringp value)
+					     (get-font value))
+					    ((consp value)
+					     (get-font-typed
+					      (car value) (cdr value)))
+					    (t value))))
 
   (define-custom-serializer 'color (lambda (value)
 				     (if (colorp value)
@@ -476,15 +485,13 @@ of choices."
 
 ;;; default groups
 
-  (defgroup focus "Focus" :require sawfish.wm.ext.auto-raise)
+  (defgroup focus "Focus")
   (defgroup move "Move/Resize" :require sawfish.wm.commands.move-resize)
   (defgroup placement "Placement")
   (defgroup appearance "Appearance")
   (defgroup workspace "Workspaces")
   (defgroup bindings "Bindings")
-  (defgroup min-max "Minimizing/Maximizing")
-  (defgroup iconify "Minimizing" :group min-max)
-  (defgroup maximize "Maximizing" :group min-max)
+  (defgroup min-max "Minimizing and Maximizing")
   (defgroup misc "Miscellaneous")
 
 
