@@ -1,5 +1,5 @@
 /* display.c -- display handling
-   $Id: display.c,v 1.44 2002/04/09 03:36:11 jsh Exp $
+   $Id: display.c 4161 2005-01-04 15:34:25Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -25,6 +25,7 @@
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 #include <X11/extensions/shape.h>
+#include <X11/Xproto.h>
 #include <string.h>
 #include <assert.h>
 #include <stdarg.h>
@@ -43,7 +44,7 @@
 # include <netdb.h>
 #endif
 
-char *prog_name, *visual_name;
+char *visual_name;
 Display *dpy;
 int screen_num, screen_width, screen_height;
 Window root_window, no_focus_window;
@@ -79,32 +80,29 @@ static int
 error_handler (Display *dpy, XErrorEvent *ev)
 {
     Lisp_Window *w;
+
 #ifdef DEBUG
     print_error (ev);
 #endif
-    if (ev->resourceid == 0)		/* probably a deleted window */
-	return 0;
-    w = x_find_window_by_id (ev->resourceid);
-    if (w != 0 && (ev->error_code == BadWindow
-		   || ev->error_code == BadDrawable))
+
+    if (ev->resourceid != 0
+	&& (ev->error_code == BadWindow || ev->error_code == BadDrawable))
     {
-	DB(("error_handler (%s)\n", rep_STR(w->name)));
-	if (!WINDOW_IS_GONE_P (w))
-	    remove_window (w, Qt, Qt);
-	/* so we call emit_pending_destroys () at some point */
-	rep_mark_input_pending (ConnectionNumber (dpy));
-	return 0;			/* ?? */
-    }
-    else
-    {
-#if 0
-	/* I'm fed up seeing these errors! :-) */
-#ifndef DEBUG
-	print_error (ev);
-#endif
-#endif
-	return 0;
+	w = x_find_window_by_id (ev->resourceid);
+
+	if (w != NULL)
+	{
+	    DB(("error_handler (%s)\n", rep_STR(w->name)));
+	    
+	    if (!WINDOW_IS_GONE_P (w))
+		remove_window (w, TRUE, TRUE);
+
+	    /* so we call emit_pending_destroys () at some point */
+	    rep_mark_input_pending (ConnectionNumber (dpy));
+	}
     }
+
+    return 0;
 }
 
 /* Installed whilst trying to set the root window event mask */
@@ -288,7 +286,7 @@ choose_visual (void)
 bool
 sys_init(char *program_name)
 {
-    char *display_name = 0;
+    char *display_name = 0, *prog_name = 0;
     repv opt;
 
 #ifdef HAVE_UNIX
