@@ -1,5 +1,5 @@
 /* events.c -- Event handling
-   $Id: events.c,v 1.154 2003/02/22 07:44:47 jsh Exp $
+   $Id: events.c 4184 2006-05-16 06:53:03Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -21,8 +21,12 @@
 
 #include "sawmill.h"
 #include <limits.h>
+#include <string.h>
 #include <time.h>
 #include <X11/extensions/shape.h>
+#ifdef HAVE_X11_EXTENSIONS_XRANDR_H
+#include <X11/extensions/Xrandr.h>
+#endif
 #include <X11/Xresource.h>
 #include <X11/Xatom.h>
 
@@ -82,6 +86,7 @@ DEFSYM(shape_notify_hook, "shape-notify-hook");
 DEFSYM(enter_frame_part_hook, "enter-frame-part-hook");
 DEFSYM(leave_frame_part_hook, "leave-frame-part-hook");
 DEFSYM(configure_request_hook, "configure-request-hook");
+DEFSYM(configure_notify_hook, "configure-notify-hook");
 DEFSYM(window_state_change_hook, "window-state-change-hook");
 
 DEFSYM(pointer_motion_threshold, "pointer-motion-threshold");
@@ -502,7 +507,7 @@ property_notify (XEvent *ev)
 		if (format == 8 && !WINDOW_IS_GONE_P (w))
 		{
 		    repv str = Qnil;
-		    if (actual == xa_compound_text)
+		    if (actual == xa_compound_text || actual == XA_STRING)
 		    {
 			char **text_list;
 			XTextProperty tprop;
@@ -520,10 +525,6 @@ property_notify (XEvent *ev)
 			    XFreeStringList(text_list);
 			}
 		    }
-		    else if (actual == XA_STRING)
-		    {
-			str = rep_string_dup (prop);
-		    }
 		    if (str == Qnil)
 			str = rep_null_string ();
 
@@ -673,9 +674,11 @@ destroy_notify (XEvent *ev)
     Lisp_Window *w = x_find_window_by_id (ev->xdestroywindow.window);
     if (w == 0 || ev->xdestroywindow.window != w->saved_id)
 	return;
-    remove_window (w, Qt, Qnil);
+    remove_window (w, TRUE, FALSE);
     property_cache_invalidate_window (rep_VAL (w));
     emit_pending_destroys ();
+    /* in case the id gets recycled but the window doesn't get gc'd..? */
+    w->saved_id = 0;
 }
 
 void
@@ -734,7 +737,7 @@ reparent_notify (XEvent *ev)
 	    && ev->xreparent.parent != w->frame)
 	{
 	    /* Not us doing the reparenting.. */
-	    remove_window (w, Qnil, Qnil);
+	    remove_window (w, FALSE, FALSE);
 	    XReparentWindow (dpy, ev->xreparent.window, ev->xreparent.parent,
 			     ev->xreparent.x, ev->xreparent.y);
 	}
@@ -753,7 +756,7 @@ map_notify (XEvent *ev)
 	if (wa.override_redirect)
 	{
 	    /* arrgh, the window changed its override redirect status.. */
-	    remove_window (w, Qnil, Qnil);
+	    remove_window (w, FALSE, FALSE);
 	}
 	else
 	{
@@ -799,7 +802,7 @@ unmap_notify (XEvent *ev)
 
 	/* Changed the window-handling model, don't let windows exist
 	   while they're withdrawn */
-	remove_window (w, Qnil, Qnil);
+	remove_window (w, FALSE, FALSE);
     }
 }
 
@@ -1100,6 +1103,29 @@ gravity_notify (XEvent *ev)
 static void
 configure_notify (XEvent *ev)
 {
+    Window window = ev->xconfigure.window;
+    Lisp_Window *w = find_window_by_id (ev->xcrossing.window);
+
+    if (window == root_window)
+    {
+        /* It will be fairly pathological case where the server
+           supports the randr extension but sawfish binary has been
+           compiled without it, but we can still respond to the
+           configureNotify event in a basic way */
+#ifdef HAVE_X11_EXTENSIONS_XRANDR_H
+        XRRUpdateConfiguration (ev);
+#endif
+        update_xinerama_info ();
+
+        screen_width = ev->xconfigure.width;
+        screen_height = ev->xconfigure.height;
+
+        Fcall_hook (Qconfigure_notify_hook, rep_LIST_1(Qroot), Qnil);
+     }
+     else if (w != 0 && w->id == window)
+     {
+        Fcall_window_hook (Qconfigure_notify_hook, rep_VAL(w), Qnil, Qnil);
+     }
 }
 
 static void
@@ -1128,9 +1154,11 @@ shape_notify (XEvent *ev)
 {
     XShapeEvent *sev = (XShapeEvent *)ev;
     Lisp_Window *w = find_window_by_id (sev->window);
-    if (w != 0 && sev->window == w->id && sev->kind == ShapeBounding)
+    if (w != 0 && sev->window == w->id
+	&& (sev->kind == ShapeBounding || sev->kind == ShapeInput))
     {
-	w->shaped = sev->shaped ? 1 : 0;
+	if (sev->kind == ShapeBounding)
+	    w->shaped = sev->shaped ? 1 : 0;
 	queue_reshape_frame (w);
 	Fcall_window_hook (Qshape_notify_hook, rep_VAL(w), Qnil, Qnil);
     }
@@ -1254,9 +1282,13 @@ deregister_event_handler (Window w)
 static inline void *
 window_event_handler (Window w)
 {
-    void *fun;
+    union {
+	void *v;
+	XPointer p;
+    } fun;
+
     return XFindContext (dpy, w, event_handler_context,
-			 (XPointer *)&fun) ? 0 : fun;
+			 &fun.p) ? 0 : fun.v;
 }
 
 
@@ -1667,6 +1699,7 @@ events_init (void)
     rep_INTERN_SPECIAL(shape_notify_hook);
     rep_INTERN_SPECIAL(enter_frame_part_hook);
     rep_INTERN_SPECIAL(leave_frame_part_hook);
+    rep_INTERN_SPECIAL(configure_notify_hook);
     rep_INTERN_SPECIAL(configure_request_hook);
     rep_INTERN_SPECIAL(window_state_change_hook);
 
