@@ -1,5 +1,5 @@
 /* functions.c -- useful window manager Lisp functions
-   $Id: functions.c,v 1.98 2002/11/03 21:00:36 jsh Exp $
+   $Id: functions.c 4174 2005-07-01 16:04:51Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -41,6 +41,7 @@
 #endif
    
 #include "sawmill.h"
+#include <string.h>
 #include <X11/Xatom.h>
 
 /* Number of outstanding server grabs made; only when this is zero is
@@ -543,14 +544,18 @@ Return the height of the root window (in pixels).
     return rep_MAKE_INT(screen_height);
 }
 
-DEFUN("sync-server", Fsync_server, Ssync_server, (void), rep_Subr0) /*
+DEFUN("sync-server", Fsync_server, Ssync_server, (repv wait), rep_Subr0) /*
 ::doc:sawfish.wm.misc#sync-server::
-sync-server
+sync-server [WAIT]
 
-Flush all pending X requests, don't wait for them to finish.
+Flush all pending X requests, don't wait for them to finish, unless WAIT
+is true.
 ::end:: */
 {
-    XFlush (dpy);
+    if (wait == Qnil)
+	XFlush (dpy);
+    else
+	XSync (dpy, False);
     return Qt;
 }
 
@@ -704,8 +709,10 @@ symbols, representing the atoms read.
 	    repv name;
 	    if (type == XA_ATOM && (name = x_atom_symbol (l_data[i])) != Qnil)
 		rep_VECTI(ret_data, i) = name;
+	    else if (type == XA_INTEGER)
+		rep_VECTI(ret_data, i) = rep_make_long_int((long) l_data[i]);
 	    else
-		rep_VECTI(ret_data, i) = rep_make_long_uint(l_data[i]);
+		rep_VECTI(ret_data, i) = rep_make_long_uint(l_data[i] & 0xffffffffUL);
 	}
 	break;
     }
@@ -932,6 +939,26 @@ Returns the numeric id of the root window of the managed screen.
 
 /* xinerama support */
 
+void
+update_xinerama_info (void)
+{
+#ifdef HAVE_X11_EXTENSIONS_XINERAMA_H
+# ifndef TEST_XINERAMA
+    if (dpy != 0)
+    {
+	if (XineramaQueryExtension (dpy, &xinerama_event_base,
+				    &xinerama_error_base))
+	{
+	    xinerama_head_info = XineramaQueryScreens (dpy, &xinerama_heads);
+	}
+    }
+# else
+    xinerama_head_info = debug_heads;
+    xinerama_heads = debug_nheads;
+# endif
+#endif
+}
+
 DEFUN ("head-count", Fhead_count, Shead_count, (void), rep_Subr0)
 {
     return rep_MAKE_INT (MAX (1, xinerama_heads));
@@ -1375,21 +1402,7 @@ functions_init (void)
     rep_mark_static (&message.font);
     rep_mark_static (&message.justify);
 
-#ifdef HAVE_X11_EXTENSIONS_XINERAMA_H
-# ifndef TEST_XINERAMA
-    if (dpy != 0)
-    {
-	if (XineramaQueryExtension (dpy, &xinerama_event_base,
-				    &xinerama_error_base))
-	{
-	    xinerama_head_info = XineramaQueryScreens (dpy, &xinerama_heads);
-	}
-    }
-# else
-    xinerama_head_info = debug_heads;
-    xinerama_heads = debug_nheads;
-# endif
-#endif
+    update_xinerama_info ();
 }
 
 void
