@@ -1,5 +1,5 @@
 /* keys.h -- Event structures
-   $Id: keys.h,v 1.10 2000/12/18 21:37:19 jsh Exp $
+   $Id: keys.h 4158 2004-11-15 18:29:19Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -37,8 +37,10 @@ enum {
     EV_CODE_MOUSE_CLICK1 = 1,
     EV_CODE_MOUSE_CLICK2,
     EV_CODE_MOUSE_MOVE,
-    EV_CODE_MOUSE_UP,
+    EV_CODE_MOUSE_UP1,
     EV_CODE_MOUSE_CLICK3,
+    EV_CODE_MOUSE_UP2,
+    EV_CODE_MOUSE_UP3,
 };
 
 /* The low 16 bits of the MODS is the standard X modifier mask. Bits
@@ -71,7 +73,7 @@ enum {
 };
 
 
-/* Support for buttons 6 and 7.
+/* Support for buttons 6, 7, 8 and 9.
 
    <X11/X.h> doesn't define these, even though XFree supports them.. */
 
@@ -89,16 +91,38 @@ enum {
 # define Button7Mask (1<<14)
 #endif
 
+#ifndef Button8
+# define Button8 8
+#endif
+#ifndef Button8Mask
+# define Button8Mask (1<<15)
+#endif
+
+#ifndef Button9
+# define Button9 9
+#endif
+#ifndef Button9Mask
+# define Button9Mask (1<<16)
+#endif
+
 #if !defined (Button6)
 # define EV_MOD_BUTTON_MASK (Button1Mask | Button2Mask | Button3Mask \
 			     | Button4Mask | Button5Mask)
 #elif !defined (Button7)
 # define EV_MOD_BUTTON_MASK (Button1Mask | Button2Mask | Button3Mask \
 			     | Button4Mask | Button5Mask | Button6Mask)
-#else
+#elif !defined (Button8)
 # define EV_MOD_BUTTON_MASK (Button1Mask | Button2Mask | Button3Mask \
 			     | Button4Mask | Button5Mask | Button6Mask \
 			     | Button7Mask)
+#elif !defined (Button9)
+# define EV_MOD_BUTTON_MASK (Button1Mask | Button2Mask | Button3Mask \
+			     | Button4Mask | Button5Mask | Button6Mask \
+                             | Button7Mask | Button8Mask)
+#else
+# define EV_MOD_BUTTON_MASK (Button1Mask | Button2Mask | Button3Mask \
+			     | Button4Mask | Button5Mask | Button6Mask \
+			     | Button7Mask | Button8Mask | Button9Mask)
 #endif
 
 
