@@ -1,5 +1,5 @@
 /* windows.c -- window manipulation
-   $Id: windows.c,v 1.127 2003/01/09 13:49:55 jsh Exp $
+   $Id: windows.c 4169 2005-02-12 06:22:18Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -21,6 +21,7 @@
 
 #include "sawmill.h"
 #include <assert.h>
+#include <string.h>
 #include <X11/extensions/shape.h>
 
 Lisp_Window *window_list;
@@ -387,6 +388,7 @@ add_window (Window id)
 	w->plist = Qnil;
 	w->frame_style = Qnil;;
 	w->icon_image = rep_NULL;
+	w->name = rep_null_string ();
 
 	/* have to put it somewhere until it finds the right place */
 	insert_in_stacking_list_above_all (w);
@@ -416,8 +418,6 @@ add_window (Window id)
 	    }
 	    XFree (prop.value);
 	}
-	if (w->name == 0)
-	    w->name = rep_null_string ();
 	w->full_name = w->name;
 	if (XGetIconName (dpy, id, &tem))
 	{
@@ -517,14 +517,14 @@ add_window (Window id)
 /* Remove W from the managed windows. If DESTROYED is nil, then the
    window will be reparented back to the root window */
 void
-remove_window (Lisp_Window *w, repv destroyed, repv from_error)
+remove_window (Lisp_Window *w, bool destroyed, bool from_error)
 {
     DB(("remove_window (%s, %s)\n",
-	rep_STR(w->name), destroyed == Qnil ? "nil" : "t"));
+	rep_STR(w->name), destroyed ? "destroyed" : "not-destroyed"));
 
     if (w->id != 0)
     {
-	if (destroyed == Qnil && from_error == Qnil)
+	if (!destroyed && !from_error)
 	{
 	    grab_window_events (w, FALSE);
 	    remove_window_frame (w);
@@ -533,13 +533,13 @@ remove_window (Lisp_Window *w, repv destroyed, repv from_error)
 	    XSetWindowBorderWidth (dpy, w->id, w->attr.border_width);
 	}
 
-	if (from_error == Qnil)
+	if (!from_error)
 	    destroy_window_frame (w, FALSE);
 
 	if (!WINDOW_IS_GONE_P (w))
 	    remove_from_stacking_list (w);
 
-	if (from_error == Qnil)
+	if (!from_error)
 	    focus_off_window (w);
 
 	w->id = 0;
@@ -547,7 +547,7 @@ remove_window (Lisp_Window *w, repv destroyed, repv from_error)
 
 	/* gc will do the rest... */
     }
-    else if (w->frame != 0 && from_error == Qnil)
+    else if (w->frame != 0 && !from_error)
 	destroy_window_frame (w, FALSE);
 }
 
@@ -1087,6 +1087,19 @@ Return t if WINDOW has been reparented to a frame window.
     return VWIN(win)->reparented ? Qt : Qnil;
 }
 
+DEFUN("window-frame-id", Fwindow_frame_id, Swindow_frame_id,
+      (repv win), rep_Subr1) /*
+::doc:sawfish.wm.windows.subrs#window-frame-id::
+window-frame-id WINDOW
+
+Return the numeric id of the framing window associated with object
+WINDOW. Returns nil if the client window has no frame.
+::end:: */
+{
+    rep_DECLARE1(win, WINDOWP);
+    return VWIN(win)->frame ? rep_MAKE_INT (VWIN(win)->frame) : Qnil;
+}
+
 DEFUN("window-id", Fwindow_id, Swindow_id, (repv win), rep_Subr1) /*
 ::doc:sawfish.wm.windows.subrs#window-id::
 window-id WINDOW
@@ -1280,26 +1293,30 @@ WINDOW. Returns the symbol `nil' if no such image.
 	   Atom actual_type;
 	   int actual_format;
 	   long nitems, bytes_after;
-	   u_long *data = 0;
+	   union {
+	       u_long *l;
+	       u_char *c;
+	   } data;
 
 	   static Atom kwm_win_icon = 0;
 
 	   if (kwm_win_icon == 0)
 	       kwm_win_icon = XInternAtom (dpy, "KWM_WIN_ICON", False);
 
+	   data.l = 0;
 	   if (XGetWindowProperty (dpy, VWIN (win)->id, kwm_win_icon,
 				   0, 2, False, kwm_win_icon,
 				   &actual_type, &actual_format,
 				   &nitems, &bytes_after,
-				   (u_char **) &data) == Success
+				   &data.c) == Success
 	       && actual_type == kwm_win_icon
 	       && bytes_after == 0)
 	   {
-	       pixmap_id = data[0];
-	       mask_id = data[1];
+	       pixmap_id = data.l[0];
+	       mask_id = data.l[1];
 	   }
-	   if (data != 0)
-	       XFree (data);
+	   if (data.l != 0)
+	       XFree (data.l);
        }
 
        VWIN (win)->icon_image = Qnil;
@@ -1547,6 +1564,7 @@ windows_init (void)
     rep_ADD_SUBR(Sshow_window);
     rep_ADD_SUBR(Swindow_visible_p);
     rep_ADD_SUBR(Swindow_framed_p);
+    rep_ADD_SUBR(Swindow_frame_id);
     rep_ADD_SUBR(Swindow_id);
     rep_ADD_SUBR(Swindow_group_id);
     rep_ADD_SUBR(Swindow_size_hints);
@@ -1621,7 +1639,7 @@ windows_kill (void)
     {
 	next = rep_VAL (w->next);
 	Fcall_window_hook (Qremove_window_hook, rep_VAL (w), Qnil, Qnil);
-	remove_window (w, Qnil, Qnil);
+	remove_window (w, FALSE, FALSE);
 	w = VWIN (next);
     }
 }
